#include <Arduino.h>
#include "IBMSureMark.h"

IBMSureMark::IBMSureMark(int RX_Pin, int TX_Pin, long baudRate) : SoftwareSerial(RX_Pin, TX_Pin) {
  	pinMode(RX_Pin, INPUT);
	pinMode(TX_Pin, OUTPUT);
begin(baudRate);
	//delay(500);
}

void IBMSureMark::setDefault() {
	//location and size.
	justify('L');
	setSize('s');
	
	//styles
	//inverseOff();
	//doubleHeightOff();
	//boldOff();
	//underlineOff();
	setFont(0x00); //takes care of styles!
	setOneEighthLineSpacing();
	setFormFeedLength(0xa0);
	
	//barcode defaults.
	setBarcodeTextParams(0x01, 0x02); //FONT B + BELOW
	setBarcodeHeight(50);
	delay(500);
}

void IBMSureMark::test() {
	println("Hello World!");
	feed(2);
}

// Location
//00 - Not printed
//01 - Above the bar code
//02 - Below the bar code
//03 - Both above and below the bar code
// Font 00 = 'A', 01 = 'B'
void IBMSureMark::setBarcodeTextParams(byte font, byte val) { 
	writeBytes(CMD_OTHER, STYLE_BARCODE_TEXT_FONT, val, CHAR_EOL);
	writeBytes(CMD_OTHER, STYLE_BARCODE_TEXT_LOCATION, val, CHAR_EOL); 
}
void IBMSureMark::setBarcodeHeight(byte val)       { writeBytes(CMD_OTHER, STYLE_BARCODE_HEIGHT, val, CHAR_EOL); }
void IBMSureMark::setBarcodeWidth(byte val)        { writeBytes(CMD_OTHER, STYLE_BARCODE_WIDTH, val, CHAR_EOL); }
/* ------------- BARCODE OPTIONS ---------------
00 - UPC-A
01 - UPC-E
02 - JAN13 (EAN)
03 - JAN8 (EAN)
04 - CODE 39
05 - ITF
06 - CODABAR
07 - CODE 128 (c)
08 - CODE 93
09 - CODE 128 (a, b)
------------------------------------------------- */
void IBMSureMark::printBarcode(char * data, byte barCodeType) {
	writeBytes(CMD_OTHER, CMD_PRINT_BARCODE, barCodeType);
	for(int i = 0; i < strlen(data); i ++){ write(data[i]); }
	write((byte)CHAR_TERM);
	delay(3000);
	feed(2);
}

//helpers
void IBMSureMark::writeBytes(uint8_t a, uint8_t b) 						 { write(a);write(b); }
void IBMSureMark::writeBytes(uint8_t a, uint8_t b, uint8_t c)            { write(a);write(b);write(c); }
void IBMSureMark::writeBytes(uint8_t a, uint8_t b, uint8_t c, uint8_t d) { write(a);write(b);write(c);write(d); }

//options are FONT_A, FONT_B, FONT_C... NOTE THAT THIS WILL RESET THE CURRENT STYLES
void IBMSureMark::setFont(byte font) {
	if (font < FONT_A) font = FONT_A; if (font > FONT_C) font = FONT_C;
	writeBytes(CMD_COMMAND, CMD_SET_PRINT_MODE, font);
}


/*			00- Code Page 437 - United States
			01 - Code Page 850 - Multilingual
			02 - Code Page 863 - Canada-French
			03 - Code Page 860 - Portugal
			04 - Code Page 865 - Norway
			05 - Printers Generic Code Page 			*/
void IBMSureMark::setCodePage(byte codepage) {
	if (codepage < 0) codepage = 0; if (codepage > 5) codepage = 5;
	writeBytes(CMD_COMMAND, CMD_SET_CODEPAGE, codepage);
}

//styles
void IBMSureMark::inverseOn()       { writeBytes(CMD_COMMAND, STYLE_INVERSE, 		0x01); }
void IBMSureMark::inverseOff()      { writeBytes(CMD_COMMAND, STYLE_INVERSE, 		0x00); }
void IBMSureMark::boldOn()          { writeBytes(CMD_COMMAND, STYLE_BOLD, 			0x01); }
void IBMSureMark::boldOff()         { writeBytes(CMD_COMMAND, STYLE_BOLD, 			0x00); }
void IBMSureMark::underlineOn()     { writeBytes(CMD_COMMAND, STYLE_UNDERLINE, 		0x01); }
void IBMSureMark::underlineOff()    { writeBytes(CMD_COMMAND, STYLE_UNDERLINE, 		0x00); }
void IBMSureMark::overlineOn()      { writeBytes(CMD_COMMAND, STYLE_OVERLINE, 		0x01); }
void IBMSureMark::overlineOff()     { writeBytes(CMD_COMMAND, STYLE_OVERLINE, 		0x00); }
void IBMSureMark::doubleHeightOn()  { writeBytes(CMD_COMMAND, STYLE_DOUBLE_HEIGHT, 	0x01); }
void IBMSureMark::doubleHeightOff() { writeBytes(CMD_COMMAND, STYLE_DOUBLE_HEIGHT,  0x00); }
void IBMSureMark::doubleWidthOn()   { writeBytes(CMD_COMMAND, STYLE_DOUBLE_WIDTH, 	0x01); }
void IBMSureMark::doubleWidthOff()  { writeBytes(CMD_COMMAND, STYLE_DOUBLE_WIDTH,	0x00); }
void IBMSureMark::upsideDownOn() 	{ writeBytes(CMD_COMMAND, STYLE_UPSIDE_DOWN,	0x01); }
void IBMSureMark::upsideDownOff() 	{ writeBytes(CMD_COMMAND, STYLE_UPSIDE_DOWN,	0x00); }
void IBMSureMark::rotatedOn() 		{ writeBytes(CMD_COMMAND, STYLE_ROTATED,		0x01); }
void IBMSureMark::rotatedOff() 		{ writeBytes(CMD_COMMAND, STYLE_ROTATED,		0x00); }

void IBMSureMark::justify(char value) {
	byte pos = 0;
	if(value == 'l' || value == 'L') pos = 0;
	if(value == 'c' || value == 'C') pos = 1;
	if(value == 'r' || value == 'R') pos = 2;
	if(value == 'x' || value == 'X') pos = 4;
	writeBytes(CMD_COMMAND, STYLE_JUSTIFICATION, pos);
}

/*	Maximum print speed for TI3/4/5/6/7/8/9 printers:
		0x00 = 51 Lines per second (DEFAULT)
		0x01 = 35 Lines per second
		0x02 = 26 Lines per second
		0x03 = 16 Lines per second
	Maximum print speed for 2CR/2NR printers:
		0x00 - 70 lines per second (DEFAULT)
		0x01 - 52 lines per second
		0x02 - 35 lines per second
		0x03 - 26 lines per second 					*/
void IBMSureMark::setMaxPrintSpeed(byte speed) {
	if (speed < 0) speed = 0; if (speed > 3) speed = 3;
	writeBytes(CMD_COMMAND, CMD_PRINT_SPEED, speed);
}

void IBMSureMark::setSize(char value) {
	byte size = 0;
	if(value == 's' || value == 'S') size = 0x00;
	if(value == 'm' || value == 'M') size = 0x11;
	if(value == 'l' || value == 'L') size = 0x22;
	if(value == 'x' || value == 'X') size = 0x44;
	writeScaleSize(size);
}

void IBMSureMark::setSize(byte w, byte h) {
	if (w < 0) w = 0;if (w > 7) w = 8;
	if (h < 0) h = 0;if (h > 7) h = 7;
	writeScaleSize((w<<4)|h);
}
void IBMSureMark::writeScaleSize(byte s) { writeBytes(CMD_OTHER, STYLE_SCALESIZE, s); } //EOL?

void IBMSureMark::tab() 		{ write(CHAR_TAB); }
void IBMSureMark::newLine() 	{ write(CHAR_EOL); }
void IBMSureMark::cutPaper() 	{ writeBytes(CMD_COMMAND, CMD_CUTPAPER, CHAR_EOL); }
void IBMSureMark::feed(byte x) 	{ writeBytes(CMD_COMMAND, CMD_PRINTFEED, x, CHAR_EOL); }
void IBMSureMark::formFeed() 	{ write(CMD_FORMFEED); }

void IBMSureMark::setCharSpacing(int spacing) 	{ writeBytes(CMD_COMMAND, STYLE_INTER_CHAR_SPACING, spacing); } // 'dot spaces'
void IBMSureMark::setOneEighthLineSpacing() 	{ writeBytes(CMD_COMMAND, STYLE_LINE_HEIGHT_ONE_EIGHTH); }
void IBMSureMark::setOneSixthLineSpacing() 		{ writeBytes(CMD_COMMAND, STYLE_LINE_HEIGHT_ONE_SIXTH); }
void IBMSureMark::setLineSpacing(byte s) 		{ writeBytes(CMD_COMMAND, STYLE_LINE_HEIGHT, (s*8)); } // in mm!
void setFormFeedLength(byte ffl)				{ writeBytes(CMD_COMMAND, CMD_SET_FORMFEED_LENGTH, ffl); } 


void IBMSureMark::printImage(byte density, int w, int h, byte * data) {
	if (w < 1) w = 1;if (w > (72*8)) w = (72*8);
	if (h < 1) h = 1;if (h > 255) h = 255;
	write(CMD_LINEFEED);
	writeBytes(CMD_COMMAND, CMD_PRINT_IMAGE);
	writeBytes(density, (byte)(w / 8), (byte)(h / 8));
	for(int i = 0; i < ((w/8) * h); i++){ write(data[i]); }
        feed(2);
}

void IBMSureMark::storeImage(byte imgId, int w, int h, byte * data) {
	if (imgId < 1) imgId = 1; if (imgId > 40) imgId = 40;
	if (w < 1) w = 1;if (w > (72*8)) w = (72*8);
	if (h < 1) h = 1;if (h > 255) h = 255;
	writeBytes(CMD_OTHER, CMD_STORE_IMAGE, imgId);
        writeBytes((byte)(w / 8), (byte)(h / 8));
	for(int i = 0; i < ((w/8) * h); i++){ write(data[i]); }
}

void IBMSureMark::printStoredImage(byte imgId, byte density) {
	if (imgId < 1) imgId = 1; if (imgId > 40) imgId = 40;
	writeBytes(CMD_OTHER, CMD_PRINT_STORED_IMAGE, density, imgId);
}

void IBMSureMark::stopNote() { playNote(0,0,0,false); }
void IBMSureMark::playNote(byte length, byte octave, byte note, bool soft) {
	if (note < 0) note = 0; if (note > 0x0b) note = 0x0b;
	if (octave < 0) octave = 0; if (octave > 0x03) octave = 0x03;
	writeBytes(CMD_COMMAND, CMD_PLAY_NOTE);
	writeBytes(length, (soft?0x80:0x00) | (octave << 4) | note);
}

void IBMSureMark::storeMessage(byte msgId, char * message) {
//      writeBytes(0x04,0x1B,0x23,0x02);
	if (msgId < 1) msgId = 1; if (msgId > 25) msgId = 25;
	writeBytes(CMD_OTHER, CMD_STORE_MESSAGE, msgId);
	for(int i = 0; i < strlen(message); i++){ write(message[i]); }
        //write(CHAR_EOL);
	writeBytes(CMD_OTHER, CMD_STORE_MESSAGE);
}
void IBMSureMark::printStoredMessage(byte msgId) {
	if (msgId < 1) msgId = 1; if (msgId > 25) msgId = 25;
	writeBytes(CMD_OTHER, CMD_PRINT_STORED_MESSAGE, msgId); //CHAR_EOL?
}

