#ifndef Thermal_h
#define Thermal_h

#include <Arduino.h>
#include <SoftwareSerial.h>

#define CMD_COMMAND 0x1b
#define CMD_OTHER   0x1d
#define CMD_PRINT_BARCODE 0x6B
#define CMD_PRINT_IMAGE 0x2a
#define CMD_STORE_IMAGE 0x2a
#define CMD_PRINT_STORED_IMAGE 0x2f
#define CMD_PLAY_NOTE 0x07
#define CMD_PRINT_SPEED 0x2f
#define CMD_SET_PRINT_MODE 0x21
#define CMD_SET_CODEPAGE 0x74
#define CMD_SET_FORMFEED_LENGTH 0x43

#define CMD_STORE_MESSAGE 0x3a
#define CMD_PRINT_STORED_MESSAGE 0x5e

#define CMD_FORMFEED 0x0c //AND CUT!
#define CMD_LINEFEED 0x0a // or 0x0d?
#define CMD_PRINTFEED 0x64 // requires third param of lines to feed.
#define CMD_CUTPAPER 0x6d

#define STYLE_BOLD 0x47
#define STYLE_INVERSE 0x48
#define STYLE_OVERLINE 0x5f
#define STYLE_UNDERLINE 0x2d
#define STYLE_SCALESIZE 0x21
#define STYLE_JUSTIFICATION 0x61
#define STYLE_UPSIDE_DOWN 0x7b
#define STYLE_DOUBLE_HEIGHT 0x68
#define STYLE_DOUBLE_WIDTH 0x57
#define STYLE_ROTATED 0x56
#define STYLE_LINE_HEIGHT 0x33
#define STYLE_LINE_HEIGHT_ONE_EIGHTH 0x31
#define STYLE_LINE_HEIGHT_ONE_SIXTH 0x32
#define DEFAULT_STYLE_INTER_CHAR_SPACING = 0x03
#define STYLE_INTER_CHAR_SPACING 0x20

#define FONT_A 0x00
#define FONT_B 0x01
#define FONT_C 0x02

#define STYLE_BARCODE_WIDTH 0x77
#define STYLE_BARCODE_HEIGHT 0x68
#define STYLE_BARCODE_TEXT_FONT 0x66
#define STYLE_BARCODE_TEXT_LOCATION 0x48

#define CHAR_TAB 0x09
#define CHAR_EOL 0x0a
#define CHAR_TERM 0x00

class IBMSureMark : public SoftwareSerial {
	public:
		IBMSureMark(int, int, long); // constructor
		void setDefault();
		void test();

		void inverseOn();
		void inverseOff();
		void doubleHeightOn();
		void doubleHeightOff();
		void doubleWidthOn();
		void doubleWidthOff();
		void boldOn();
		void boldOff();
		void underlineOn();
		void underlineOff();
		void overlineOn();
		void overlineOff();
		void upsideDownOn();
		void upsideDownOff();
		void rotatedOn();
		void rotatedOff();
		void justify(char value);

		void setFont(byte font);
		void setCodePage(byte codepage);
		void setOneSixthLineSpacing();
		void setOneEighthLineSpacing();
		void setCharSpacing(int spacing);
		void setLineSpacing(byte s);
		void setSize(char value);
		void setSize(byte w, byte h);
		void setLineHeight(int val = 32);
		void setFormFeedLength(byte ffl);

		void printBarcode(char * data, byte barCodeType);
		void setBarcodeHeight(byte val);
		void setBarcodeWidth(byte val);
		void setBarcodeTextParams(byte font, byte val);

		void printImage(byte density, int w, int h, byte * data);
		void storeImage(byte imgId, int w, int h, byte * data);
		void printStoredImage(byte imgId, byte density);

		void tab();
		void newLine();
		void cutPaper();
		void feed(byte x = 1);
		void formFeed();
		
		void playNote(byte length, byte octave, byte note, bool soft);
		void stopNote();
		
		void storeMessage(byte msgId, char * message);
		void printStoredMessage(byte msgId);

	private:
		void writeBytes(uint8_t a, uint8_t b);
		void writeBytes(uint8_t a, uint8_t b, uint8_t c);
		void writeBytes(uint8_t a, uint8_t b, uint8_t c, uint8_t d);
		void writeScaleSize(byte s);
};

#endif
